/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER             "0.1.1"
#define BUFFSZ          8192
#define PASS            "PassWd="
#define B64DECX         bb = *bf; \
                        for(; \
                        bb && \
                        ((bb == ' ') || \
                        (bb == '\t') || \
                        (bb == '\r') || \
                        (bb == '\n')); \
                        bf++);



#ifndef WIN32
    #define stristr strcasestr
#endif

#ifndef u_char
    #define u_char  unsigned char
#endif



void becky_pwd(u_char *pwd, int len);
u_char *Base64Decode(u_char *buf, u_char *ret);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    int     len;
    u_char  buff[BUFFSZ + 1],
            *ptr;


    setbuf(stdout, NULL);

    fputs("\n"
        "Becky! Internet mail passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <Mailbox.ini>\n"
            "\n"
            "The file is located in Default\\ID.mb where ID is a number that identify the mailbox\n"
            "\n", argv[0]);
        fputs("\n    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("\nOpening file %s\n\n", argv[1]);
    fd = fopen(argv[1], "rt");
    if(!fd) std_err();

    while(fgets(buff, BUFFSZ, fd)) {
        ptr = stristr(buff, PASS);
        if(ptr) {
            ptr += sizeof(PASS) - 1;
            len = strlen(ptr) - 1;
            becky_pwd(ptr, len);
            printf("%s\n", buff);
        } else {
            fputs(buff, stdout);
        }
    }

    fclose(fd);
    fputs("\n\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void becky_pwd(u_char *pwd, int len) {
    u_char  *ptr = pwd;

    while(len--) *ptr++ ^= 2;
    Base64Decode(pwd, pwd);
    Base64Decode(pwd, pwd);
}



u_char *Base64Decode(u_char *buf, u_char *ret) {
    int     c,
            d,
            i,
            *Base;
    u_char  bb,
            *bf,
            *rf;

    if(!*buf) {
        *ret = 0;
        return(ret);
    }

    Base = (int *)malloc(sizeof(int) << 8);
    if(!Base) {
        strcpy(ret, buf);
        return(ret);
    }
    for(i = 0; i < 26; i++) {
        Base['A' + i] = i;
        Base['a' + i] = i + 26;
    }
    for(i = 0; i < 10; i++) {
        Base['0' + i] = i + 52;
    }
    Base['+'] = 62;
    Base['/'] = 63;

    bf = buf;
    rf = ret;
    B64DECX;
    while(1) {
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        d = Base[bb];
        *rf++ = (c << 2) | (d >> 4);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];
        *rf++ = ((d & 0xF) << 4) | (c >> 2);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        *rf++ = ((c & 0x3) << 6) | Base[bb];
    }
    *rf = 0;
    free(Base);
    return(rf);
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}

